from z3c.form.form import EditForm
from z3c.form.field import Fields

from z3c.feature.core.python import PythonInterpreterFeature
from z3c.feature.core.python import ScriptFeature
from z3c.feature.core import interfaces


class PythonInterpreterWebView(EditForm):
    fields = Fields(interfaces.IPythonInterpreterFeature).select('executableName')

class PythonInterpreterWebFeature(object):
    viewFactory = PythonInterpreterWebView
    contentFactory = PythonInterpreterFeature
    title = u'Python Interpreter'
    description = (u"Adds a Python interpreter with all the path "
                   u"information needed to make project code available.")


class ScriptWebView(EditForm):

    fields = Fields(interfaces.IScriptFeature)

    def update(self):
        super(ScriptWebView, self).update()
        self.label = self.context.featureTitle

class ScriptWebFeature(object):
    viewFactory = ScriptWebView
    contentFactory = ScriptFeature
    title = u'Console Script'
    description = (u"Adds an entry point for setuptools to create a "
                   u"console script that runs a python function. "
                   u"Great for projects that run from the command line "
                   u"and require insanely easy installation.")



from z3c.form.form import EditForm
from z3c.form.field import Fields

from z3c.feature.core.unittest import TestingFeature
from z3c.feature.core import interfaces


class TestingWebView(EditForm):
    fields = Fields(interfaces.ITestingFeature)


class TestingWebFeature(object):
    viewFactory = TestingWebView
    contentFactory = TestingFeature
    title = u"Automated Tests"
    description = (u"Adds all the boiler plate for running an "
                   u"automated doctest suite along with test "
                   u"coverage reporting.")



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Feature interfaces

$Id: interfaces.py 98421 2009-03-27 12:42:04Z pcardune $
"""
import datetime
import os

import zope.interface
from zope.interface.interfaces import IInterface
import zope.schema

from z3c.builder.core.interfaces import troveClassiferVocabulary

FEATURE_GROUP = 'z3c.feature'

class IFeatureType(IInterface):
    """An interface that describes a particular type of feature."""

class DuplicateFeatureConflictError(ValueError):

    def __init__(self, type):
        self.type = type
        ValueError.__init__(self, type)

class CyclicDependencyError(ValueError):

    def __init__(self, cycle):
        self.cycle = cycle
        ValueError.__init__(self, cycle)

class MissingFeatureDependencyError(KeyError):
    def __init__(self, feature, dependencyName):
        self.feature = feature
        self.dependencyName = dependencyName
        KeyError.__init__(self, dependencyName)

    def __str__(self):
        return ('Feature "%s" depends on "%s" but no "%s" '
                'feature was specified') % (self.feature.featureTitle,
                                            self.dependencyName,
                                            self.dependencyName)

class IFeatureSchema(IInterface):
    """A schema describing the configuration parameters of a feature."""

class IBaseFeatureSchema(zope.interface.Interface):
    """A trivial feature schema."""
zope.interface.directlyProvides(IBaseFeatureSchema, IFeatureSchema)


class IFeature(zope.interface.Interface):
    """An object representing a possible feature."""

    featureTitle = zope.schema.TextLine(
        title=u"Feature Title",
        description=u"A user readable title for this feature.")

    featureDocumentation = zope.schema.Text(
        title=u"Feature Documentation",
        description=u"""ReSTructured text documentation on the feature.

        This should explain what files were modified/created by the
        feature and what each piece does, with tips on modifying them
        later.  Documentation for all the features are combined into
        a single README.txt in the project Root.
        """)

    featureSingleton = zope.schema.Bool(
        title=u"Feature Sigelton",
        description=(u"When set, only one instance of this feature can "
                     u"be applied to the context."),
        default=True)

    featureDependencies = zope.schema.Tuple(
        title=u"Feature Dependencies",
        description=(u"A list of names of features that this feature "
                     u"depends on."),
        value_type=zope.schema.ASCIILine())

    def update(features=()):
        """Update the feature.

        Optionally, all other features that are applied are passed into the
        method as an argument.
        """

    def applyTo(context):
        """Apply the given feature to the context."""

    def findEntryPoint():
        """Returns the egg name and entry point name.

        If no entry point is found, `(None, None)` is returned.
        """

    def toXML(asString=False, prettyPrint=False):
        """Returns an etree node object representing the feature in xml.

        Optional keyword arguments include asString and prettyPrint
        which if set to true will return an xml string that is pretty
        printed.
        """

class IHaveAppliedFeatures(zope.interface.Interface):
    """A component that keeps track of the features applied to it."""

    appliedFeatures = zope.schema.List(
        title=u'Applied Features',
        description=u'A list of features that were applied on the builder.')


class IProjectTemplateProvider(zope.interface.Interface):
    """Object that provides a projecte template."""

    title = zope.schema.TextLine(
        title=u"Title",
        description=u"A title that can be provided to the user.")

    description = zope.schema.TextLine(
        title=u"Description",
        description=u"A description that can be provided to the user.")

    def getFeatures():
        """Returns a dictionary of entrypoint name to features that
           are part of this template."""


class IFileBasedTemplateProvider(IProjectTemplateProvider):

    filename = zope.schema.TextLine(
        title=u"Filename")


class IMetaDataFeature(zope.interface.Interface):

    license = zope.schema.TextLine(
        title=u'License',
        default=u'GNU General Public License (GPL)',
        required=False)

    version = zope.schema.TextLine(
        title=u'Version',
        description=u'An initial version number for the project',
        default=u'0.1.0',
        required=False)

    description = zope.schema.TextLine(
        title=u'Project Description',
        description=u'Short Description of the project',
        required=False)

    author = zope.schema.TextLine(
        title=u'Author(s)',
        description=u'Name of the project author(s)',
        required=False)

    author_email = zope.schema.TextLine(
        title=u'Author Email',
        description=u'Email address for the project author(s)',
        required=False)

    keywords = zope.schema.List(
        title=u'Keywords',
        description=u'A list of keywords related to the project, one per line.',
        value_type=zope.schema.TextLine(),
        required=False)

    url = zope.schema.TextLine(
        title=u'URL',
        description=(u'The url for the project. Defaults to '
                     u'http://pypi.python.org/pypi/[project-name]'),
        required=False)

    classifiers = zope.schema.Set(
        title=u"Trove Classifiers",
        value_type=zope.schema.Choice(
            vocabulary=troveClassiferVocabulary),
        required=False)

    commentHeader = zope.schema.Text(
        title=u'Comment Header',
        description=(u'A comment header that should appear at the top of every '
                     u'python file (like a license header).'),
        required=False)

    namespace_packages = zope.schema.List(
        title=u'Namespace Packages',
        description=(u'A list of namespace packages that should be created, one '
                     u'per line (i.e. zope or zc or z3c or collective)'),
        value_type=zope.schema.TextLine(),
        required=False)

    install_requires = zope.schema.List(
        title=u"Install Requires",
        description=(u"A list of additional dependencies, one per line "
                     u"(i.e. lxml)"),
        value_type=zope.schema.TextLine(),
        required=False)

    extras_require = zope.schema.Dict(
        title=u"Extras Require",
        key_type=zope.schema.TextLine(),
        value_type=zope.schema.List(value_type=zope.schema.TextLine()))

zope.interface.directlyProvides(IMetaDataFeature, IFeatureSchema)


class ICommentHeaderZPLFeature(zope.interface.Interface):

    author = zope.schema.TextLine(
        title=u'Author(s)',
        description=u'Name of the project author(s)',
        required=False)

    year = zope.schema.Int(
        title=u"Year",
        description=u'The copyright year',
        default=datetime.date.today().year)

zope.interface.directlyProvides(ICommentHeaderZPLFeature, IFeatureSchema)


class IProprietaryHeaderFeature(zope.interface.Interface):

    year = zope.schema.Int(
        title=u"Year",
        description=u'The copyright year',
        default=datetime.date.today().year)

    company = zope.schema.TextLine(
        title=u'Company',
        description=u'The name of your company, (i.e. Foo Inc.)')

    location = zope.schema.TextLine(
        title=u'Location',
        description=u'Location of your company (i.e. San Francisco, USA)')

zope.interface.directlyProvides(IProprietaryHeaderFeature, IFeatureSchema)


class IPythonInterpreterFeature(zope.interface.Interface):

    executableName = zope.schema.ASCIILine(
        title=u'Executable',
        description=(u'The path to the python executable. '
                     u'(i.e. /opt/python2.6/bin/python or just python)'),
        default='python')

zope.interface.directlyProvides(IPythonInterpreterFeature, IFeatureSchema)


class ITestingFeature(zope.interface.Interface):

    coverageDirectory = zope.schema.TextLine(
        title=u"Coverage Directory",
        description=u"Directory where test coverage data should be placed.",
        default=u"${buildout:directory}/coverage")

    coverageReportDirectory = zope.schema.TextLine(
        title=u'Coverage Report Directory',
        description=u'Directory where coverage reports should be generated',
        default=u'${buildout:directory}/coverage/report')

zope.interface.directlyProvides(ITestingFeature, IFeatureSchema)


class IScriptFeature(zope.interface.Interface):

    scriptName = zope.schema.TextLine(
        title=u'Script Name',
        description=(u'The name of the script that will '
                     u'be made available on the command line. '
                     u'Defaults to the name of the project.'),
        required=False)

    scriptFile = zope.schema.TextLine(
        title=u'Script File',
        description=u'The file where the script will be located',
        default=u"script.py")

    scriptFunction = zope.schema.TextLine(
        title=u'Script Function',
        description=u'The function in the script file that runs the script',
        default=u"main")

zope.interface.directlyProvides(IScriptFeature, IFeatureSchema)


class IDocumentationFeature(zope.interface.Interface):

    layoutTemplatePath = zope.schema.URI(
        title=u'Layout Template',
        description=u"URL for a layout template.  Leave blank for default",
        required=False)

    cssPath = zope.schema.URI(
        title=u'CSS url',
        description=(u'URL for a css file that should be used.  Leave blank '
                     u'for default.'),
        required=False)

    additionalEggs = zope.schema.List(
        title=u'Additional Packages',
        description=(u'Additional packages for which documentation should '
                     u'be built'),
        value_type=zope.schema.TextLine(),
        required=False)

zope.interface.directlyProvides(IDocumentationFeature, IFeatureSchema)



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Core Python Features

$Id: python.py 98421 2009-03-27 12:42:04Z pcardune $
"""
import zope.interface
from zope.schema.fieldproperty import FieldProperty

from z3c.builder.core import project, buildout
from z3c.builder.core.python import ModuleBuilder, FunctionBuilder
from z3c.feature.core import base, interfaces, xml


PYTHON_INTERPRETER_DOCUMENTATION = """
The Python Interpreter feature creates an alias to whatever python
interpreter you want inside your project's bin/ directory.
Furthermore, it adds your project and all its egg dependencies to the
python path.  After running buildout from your project directory you
can start up the interpreter as normal::

  $ ./bin/python

  >>> from myproject import mymodule
  >>> form a.dependency import something
"""

SCRIPT_FEATURE_DOCUMENTATION = """
The Command Line Script Feature exposes a python function in your
project as a command line script.  There are several pieces to this:

  #. **The script file.**

        There is a script file located at %(scriptFile)s with a function
        in it called %(scriptFunction)s.  This is what you should modify to
        make your script actually do something.

  #. **Setuptools entry point.**

       When someone installs your project using setuptools, for example
       with the command::

         $ easy_install yourproject

       any entry points in the console_script group will turn into
       executable scripts that the end user can just run.  This make your
       project into an application and not just a set of python
       libraries.

       The entry point is created by modifying the ``setup.py`` file.
       Look for the keyword parameter called entry_points.

  #. ``scripts`` **buildout part.**

       Finally there is also a part added to buildout.cfg that makes the
       script defined in the entry point available in the projects bin/
       directory.  This is only for development purposes.
"""

class PythonInterpreterFeature(base.BaseFeature):
    zope.interface.implements(interfaces.IPythonInterpreterFeature)

    executableName = FieldProperty(
        interfaces.IPythonInterpreterFeature['executableName'])

    featureTitle = u'Python Interpreter'
    featureDocumentation = PYTHON_INTERPRETER_DOCUMENTATION

    def _applyTo(self, context):
        pythonPartBuilder = buildout.PartBuilder(u'python')
        pythonPartBuilder.addValue('recipe', 'zc.recipe.egg')
        pythonPartBuilder.addValue('interpreter', self.executableName)
        pythonPartBuilder.addValue('eggs', context.name)
        context.buildout.add(pythonPartBuilder)


class ScriptFeature(base.BaseFeature):
    zope.interface.implements(interfaces.IScriptFeature)

    scriptName = FieldProperty(interfaces.IScriptFeature['scriptName'])
    scriptFile = FieldProperty(interfaces.IScriptFeature['scriptFile'])
    scriptFunction = FieldProperty(interfaces.IScriptFeature['scriptFunction'])

    featureTitle = u'Command Line Script'

    @property
    def featureDocumentation(self):
        return SCRIPT_FEATURE_DOCUMENTATION % dict(
            scriptFunction=self.scriptFunction,
            scriptFile=self.scriptFile)

    def _applyTo(self, context):
        scriptName = self.scriptName or context.name
        moduleBuilder = ModuleBuilder(self.scriptFile)
        moduleBuilder.add(FunctionBuilder(
            self.scriptFunction,
            kwargs={'args':None},
            docstring=('Runs the %s script '
                       'from the %s project') % (scriptName, context.name),
            code='print "Successfully ran the %s script"' % scriptName
            ))

        context.package.add(moduleBuilder)

        context.setup.addEntryPoints(
            'console_scripts', ['%s = %s.%s:%s' % (scriptName,
                                                   context.name,
                                                   self.scriptFile[:-3],
                                                   self.scriptFunction)])

        scriptsPart = buildout.PartBuilder(u'scripts')
        scriptsPart.addValue('recipe','zc.recipe.egg:scripts')
        scriptsPart.addValue('eggs', context.name)
        scriptsPart.addValue('script', scriptName)
        context.buildout.add(scriptsPart)



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Base Classes Feature

$Id: base.py 98421 2009-03-27 12:42:04Z pcardune $
"""
import lxml.etree
import pkg_resources
import zope.interface
import zope.schema
from zope.schema.fieldproperty import FieldProperty
from z3c.feature.core import interfaces, xml

MISSING_DOCUMENTATION = """
Sorry, but the authors of the \"%s\" feature are
big meanies who don't like to make software
accessible."""

class BaseFeature(object):
    zope.interface.implements(
        interfaces.IFeature, interfaces.IBaseFeatureSchema)

    featureSingleton = FieldProperty(interfaces.IFeature['featureSingleton'])
    featureDependencies = ()

    @property
    def featureTitle(self):
        """See ``z3c.feature.core.interfaces.IFeature``"""
        return self.__class__.__name__

    @property
    def featureDocumentation(self):
        """See ``z3c.feature.core.interfaces.IFeature``"""
        return MISSING_DOCUMENTATION  % self.featureTitle

    @classmethod
    def fromXMLNode(cls, node, omit=()):
        feature = cls()
        schema = getFeatureSchema(feature)
        data = xml.extractData(node, schema)
        for fieldName in schema:
            if fieldName in omit:
                continue
            value = data.get(fieldName)
            if value:
                setattr(feature, fieldName, value)
        return feature

    @classmethod
    def fromXML(cls, xml):
        tree = lxml.etree.fromstring(xml)
        return cls.fromXMLNode(tree)

    def findEntryPoint(self):
        set = pkg_resources.working_set
        for entryPoint in set.iter_entry_points(interfaces.FEATURE_GROUP):
            if entryPoint.load() == self.__class__:
                return entryPoint.dist.project_name, entryPoint.name
        return None, None

    def toXML(self, asString=False, prettyPrint=False):
        feature = lxml.etree.Element('feature')
        egg, name = self.findEntryPoint()
        feature.set('type', egg + ':' + name if egg and name else 'unknown')
        schema = getFeatureSchema(self)
        for fieldName in zope.schema.getFields(schema):
            if fieldName in interfaces.IFeature:
                continue

            value = getattr(self, fieldName)
            if value != schema[fieldName].default:
                featureOption = lxml.etree.SubElement(feature, fieldName)
                if isinstance(value, (list, tuple, set)):
                    for item in value:
                        itemElem = lxml.etree.SubElement(featureOption, 'item')
                        itemElem.text = str(item)
                else:
                    featureOption.text = str(value)
        if asString:
            return lxml.etree.tostring(feature, pretty_print=prettyPrint)
        return feature

    def update(self, features=None):
        """See ``z3c.feature.core.interfaces.IFeature``"""

    def _applyTo(self, context):
        pass

    def applyTo(self, context):
        """See ``z3c.feature.core.interfaces.IFeature``"""
        if interfaces.IHaveAppliedFeatures.providedBy(context):
            context.appliedFeatures.append(self)
        self._applyTo(context)

    def __repr__(self):
        return "<%s %r>" % (self.__class__.__name__, self.featureTitle)


def getFeatureSchema(feature):
    if isinstance(feature, type):
        ifaces = zope.interface.implementedBy(feature).flattened()
    else:
        ifaces = zope.interface.providedBy(feature).flattened()
    for iface in ifaces:
        if interfaces.IFeatureSchema.providedBy(iface):
            return iface
    return None


def getFeatureTypes(feature):
    return [iface
            for iface in zope.interface.providedBy(feature).flattened()
            if interfaces.IFeatureType.providedBy(iface)]


def resolveDependencies(features, resolution=None, seen=None, types=None,
                        all=None):
    # List of all features.
    if all is None:
        all = features
    # List of all feature types that have been found on singletons.
    if types is None:
        types = []
    # List of features that have been resolved in the correct order.
    if resolution is None:
        resolution = []
    # A list of seen features for a particular dependency sub-path.
    if seen is None:
        seen = []
    # Loop through all features and resolve them.
    for name, feature in features.items():
        # If the feature is a singleton record its types.
        if feature.featureSingleton:
            for type in getFeatureTypes(feature):
                if type in types:
                    raise interfaces.DuplicateFeatureConflictError(type)
                types.append(type)
        # If a feature is already resolved, skip the feature.
        if feature in resolution:
            continue
        # If we have seen the feature already, we have a cycle.
        if feature in seen:
            raise interfaces.CyclicDependencyError(seen[seen.index(feature):])
        # If we do not have a cycle, add the feature to the list of seen ones
        seen.append(feature)
        # Resolve the dependencies of all children.
        if feature.featureDependencies:
            try:
                deps = dict(
                    [(name, all[name]) for name in feature.featureDependencies])
            except KeyError:
                raise interfaces.MissingFeatureDependencyError(feature, name)
            resolveDependencies(deps, resolution, seen, types, all)
        # Add the feature to the resolution.
        if  feature not in resolution:
            resolution.append(feature)
        # Remove the feature from the current dependency path.
        seen.pop()

    return resolution

def applyFeatures(features, project):
    """Apply a list of features to the project."""
    # Make sure the project collects all applied features
    zope.interface.directlyProvides(project, interfaces.IHaveAppliedFeatures)
    project.appliedFeatures = []
    # Apply features one by one.
    featureDict = dict(
        [(feature.findEntryPoint()[1] or 'unknwon-%i' %idx, feature)
         for idx, feature in enumerate(features)])
    for feature in resolveDependencies(featureDict):
        feature.update(featureDict)
        feature.applyTo(project)


class FileBasedTemplateBase(object):

    _filename = None

    def getFeatures(self):
        if self._filename is None:
            raise ValueError("Missing filename attribute.")
        return xml.getFeatures(open(self._filename))


def FileBasedTemplate(filename, title, description):
    FileBasedTemplate = type(
        '<FileBasedTemplate for %s' % filename,
        (FileBasedTemplateBase,),
        dict(title=title,
             description=description,
             _filename=filename))

    zope.interface.classImplements(
        FileBasedTemplate, interfaces.IProjectTemplateProvider)
    return FileBasedTemplate



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Meta Data Feature

$Id: metadata.py 98421 2009-03-27 12:42:04Z pcardune $
"""
import os
import types
import zope.interface
from zope.schema.fieldproperty import FieldProperty

from z3c.builder.core import project, buildout
from z3c.builder.core.base import getTemplatePath, FileBuilder
from z3c.builder.core.interfaces import IFileBuilder
from z3c.feature.core import base, interfaces, xml


METADATA_FEATURE_DOCUMENTATION = """
The Metadata feature sets up the setup.py file which is what
setuptools uses to generate distributable python eggs.
"""

COMMENT_HEADER_ZPL_DOCUMENTATION = """
The Zope Public License file header looks something like this::

%(header)s
"""

PROPRIETARY_HEADER_DOCUMENTATION = """
The proprietary header looks something like this::

%(header)s
"""

DOCUMENTATION_FEATURE_DOCUMENTATION = """
The ReSTructured Text Documentation feature hooks up scripts
for generating html (or latex for that matter) documentation
from ReSTructured text files using Sphinx.  There are a few
pieces involved in this hookup:

  #. ``buildout.cfg`` **part section**

       This looks something like::

         [docs]
         recipe = z3c.recipe.sphinxdoc
         eggs = yourproject [docs]
                z3c.recipe.sphinxdoc
         default.css =
         layout.html =

       This buildout part section will generate a script in the
       bin directory called ``docs`` which you can run liket his::

         $ ./bin/docs

       Documentation will be put into the ``parts/docs/`` directory, with
       one directory for each package specified in the eggs parameter of
       the docs section.  See the documentation for z3c.recipe.sphinxdoc
       for more information.  It can be found at

         http://pypi.python.org/pypi/z3c.recipe.sphinxdoc

  #. ``setup.py extras_require`` **section**

       For the docs to build correctly, there must be a ``docs`` section in
       ``extras_require`` that pulls in the Sphinx dependencies.

  #. ``index.txt`` **file in project src**

       An ``index.txt`` file will be added to the src directory.  This
       serves as the root document used by Sphinx to generate all the
       documentation.  See the Sphinx documentation for what sort of things
       you can put in here at

         http://sphinx.pocoo.org/
"""

class MetaDataFeature(base.BaseFeature):
    zope.interface.implements(interfaces.IMetaDataFeature)

    version = FieldProperty(interfaces.IMetaDataFeature['version'])
    license = FieldProperty(interfaces.IMetaDataFeature['license'])
    url = FieldProperty(interfaces.IMetaDataFeature['url'])
    classifiers = FieldProperty(interfaces.IMetaDataFeature['classifiers'])
    keywords = FieldProperty(interfaces.IMetaDataFeature['keywords'])
    author = FieldProperty(interfaces.IMetaDataFeature['author'])
    author_email = FieldProperty(interfaces.IMetaDataFeature['author_email'])
    description = FieldProperty(interfaces.IMetaDataFeature['description'])
    commentHeader = FieldProperty(interfaces.IMetaDataFeature['commentHeader'])
    namespace_packages = FieldProperty(
        interfaces.IMetaDataFeature['namespace_packages'])
    install_requires = FieldProperty(
        interfaces.IMetaDataFeature['install_requires'])
    extras_require = FieldProperty(interfaces.IMetaDataFeature['extras_require'])

    featureTitle = u'Metadata'
    featureDocumentation = METADATA_FEATURE_DOCUMENTATION

    @classmethod
    def fromXMLNode(cls, node, omit=()):
        feature = super(MetaDataFeature, cls).fromXMLNode(
            node,
            omit=('classifiers', 'extras_require'))

        schema = base.getFeatureSchema(feature)
        data = xml.extractData(node, schema)
        classifiers = data.get('classifiers')

        feature.classifiers = set()
        if classifiers:
            feature.classifiers = [c.strip() for c in classifiers.split('\n')]
        return feature

    def _applyTo(self, context):
        """See ``z3c.feature.core.interfaces.IFeature``"""
        # Assign all meta data to setup.
        for name in ('version', 'license', 'url', 'keywords', 'author',
                     'author_email', 'description', 'namespace_packages',
                     'install_requires','extras_require'):
            value = getattr(self, name)
            if value is not None:
                setattr(context.setup, name, value)
        # Set the comment header.
        if self.commentHeader is not None:
            context.commentHeader = self.commentHeader


class CommentHeaderZPLFeature(base.BaseFeature):
    zope.interface.implements(interfaces.ICommentHeaderZPLFeature)

    _template = unicode(open(getTemplatePath('zpl-header.py')).read())

    year = FieldProperty(interfaces.ICommentHeaderZPLFeature['year'])
    author = FieldProperty(interfaces.ICommentHeaderZPLFeature['author'])

    featureTitle = u'ZPL Header'

    @property
    def featureDocumentation(self):
        """See ``z3c.feature.core.interfaces.IFeature``"""
        header = '  ' + self.renderCommentHeader().replace('\n', '\n  ')
        return COMMENT_HEADER_ZPL_DOCUMENTATION % {'header':header}

    def renderCommentHeader(self):
        return self._template % dict(year=self.year,
                                     author=self.author)

    def _applyTo(self, context):
        """See ``z3c.feature.core.interfaces.IFeature``"""
        context.commentHeader = self.renderCommentHeader()


class ProprietaryHeaderFeature(base.BaseFeature):
    zope.interface.implements(interfaces.IProprietaryHeaderFeature)

    _template = unicode(open(getTemplatePath('proprietary-header.py')).read())

    year = FieldProperty(interfaces.IProprietaryHeaderFeature['year'])
    company = FieldProperty(interfaces.IProprietaryHeaderFeature['company'])
    location = FieldProperty(interfaces.IProprietaryHeaderFeature['location'])

    featureTitle = u'Proprietary Header'

    @property
    def featureDocumentation(self):
        """See ``z3c.feature.core.interfaces.IFeature``"""
        header = '  ' + self.renderCommentHeader().replace('\n', '\n  ')
        return PROPRIETARY_HEADER_DOCUMENTATION % {'header':header}

    def renderCommentHeader(self):
        return self._template % dict(year=self.year,
                                     company=self.company,
                                     location=self.location)

    def _applyTo(self, context):
        """See ``z3c.feature.core.interfaces.IFeature``"""
        context.commentHeader = self.renderCommentHeader()


class DocumentationFileBuilder(FileBuilder):

    template = os.path.join(
        os.path.dirname(__file__), 'file-templates', 'index-template.txt')

    def update(self):
        project = self.getProject()
        self.title = project.name + ' Documentation'
        self.TOC = []
        for builder in self.__parent__.values():
            if (IFileBuilder.providedBy(builder) and
                builder.filename.endswith('.txt') and
                builder is not self):

                self.TOC.append(builder.filename[:-4])

    def render(self):
        heading = '%(border)s\n%(title)s\n%(border)s' % dict(
            border='='*len(self.title),
            title=self.title)
        TOC = '\n   '.join(self.TOC)
        return open(self.template).read() % dict(heading=heading,
                                                 TOC=TOC)

class DocumentationFeature(base.BaseFeature):
    zope.interface.implements(interfaces.IDocumentationFeature)

    layoutTemplatePath = FieldProperty(
        interfaces.IDocumentationFeature['layoutTemplatePath'])
    cssPath = FieldProperty(
        interfaces.IDocumentationFeature['cssPath'])
    additionalEggs = FieldProperty(
        interfaces.IDocumentationFeature['additionalEggs'])

    featureTitle = u'Restructured Text Documentation'
    featureDocumentation = DOCUMENTATION_FEATURE_DOCUMENTATION

    def _applyTo(self, context):
        """See ``z3c.feature.core.interfaces.IFeature``"""
        docsPart = buildout.PartBuilder(u'docs')
        docsPart.addValue('recipe', 'z3c.recipe.sphinxdoc')
        eggs = [context.name+' [docs]'] + (self.additionalEggs or [])
        eggsString = 'z3c.recipe.sphinxdoc'
        for egg in eggs:
            eggsString += '\n       '+egg
        docsPart.addValue('eggs', eggsString)
        docsPart.addValue('layout.html', self.layoutTemplatePath or '')
        docsPart.addValue('default.css', self.cssPath or '')
        context.buildout.add(docsPart)

        context.setup.addExtrasRequires('docs', ['Sphinx'])

        indexBuilder = DocumentationFileBuilder(u'index.txt')
        context.package.add(indexBuilder)



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Core Python Features

$Id: template.py 98421 2009-03-27 12:42:04Z pcardune $
"""
import os
import pkg_resources
from zope.interface import classImplements
from z3c.feature.core import interfaces, xml


class FileBasedTemplateBase(object):

    filename = None

    def getFeatures(self):
        if self.filename is None:
            raise ValueError("Missing filename attribute.")
        return xml.getFeaturesDict(open(self.filename))


def FileBasedTemplate(filename, title, description):
    FileBasedTemplate = type(
        '<FileBasedTemplate for %s' % filename,
        (FileBasedTemplateBase,), dict(title=title,
                                   description=description,
                                   filename=filename))

    classImplements(FileBasedTemplate, interfaces.IFileBasedTemplateProvider)
    return FileBasedTemplate


def getTemplateList():
    names = set()
    templates = {}
    for entryPoint in pkg_resources.working_set.iter_entry_points('z3c.boiler.template'):
        template = entryPoint.load()()
        if entryPoint.name in names:
            name = entryPoint.dist.project_name+':'+entryPoint.name
        else:
            name = entryPoint.name
        templates[name] = template
    return templates


def getTemplate(name):
    return getTemplateList()[name]

CommandLineProjectTemplate = FileBasedTemplate(
    os.path.join(os.path.dirname(__file__),"command-line-project.xml"),
    u"Command Line Program",
    u"Includes all the features you would want for a command line program.")

PythonPackageProjectTemplate = FileBasedTemplate(
    os.path.join(os.path.dirname(__file__),"python-package-project.xml"),
    u"Python Package",
    u"Just a simple python package with few bells and whistles.")



import unittest
import zope.testing.doctest

def test_suite():
    return unittest.TestSuite((

        zope.testing.doctest.DocFileSuite(
            '../README.txt',
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),


        ))



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""XML Processing

$Id: xml.py 98421 2009-03-27 12:42:04Z pcardune $
"""

from lxml import etree
import types
import pkg_resources
from zc.buildout import easy_install
from zope.interface import directlyProvides

from z3c.builder.core import project, base
from z3c.feature.core import interfaces

target_dir = pkg_resources.working_set.find(
    pkg_resources.Requirement.parse('zc.buildout')).location


class FeatureDocBuilder(base.FileBuilder):
    """A builder to document all applied features of a project."""

    def render(self):
        result = open(
            base.getTemplatePath('zboiler-doc-header.txt'), 'r').read()
        for feature in self.getProject().appliedFeatures:
            title = feature.featureTitle
            result += "\n\n" + title + "\n"
            result += "-"*len(title) + "\n"
            docs = feature.featureDocumentation.strip()
            result += "\n" + docs + "\n\n"
        return result


def getNode(nodeOrFileOrStr):
    if isinstance(nodeOrFileOrStr, types.StringTypes):
        return etree.fromstring(nodeOrFileOrStr)
    elif hasattr(nodeOrFileOrStr, 'read'):
        data = nodeOrFileOrStr.read()
        nodeOrFileOrStr.seek(0)
        return etree.fromstring(data)
    elif isinstance(nodeOrFileOrStr, etree._Element):
        return nodeOrFileOrStr
    else:
        raise TypeError("Could not get a lxml.etree.Element "
                        "object from %s" % nodeOrFileOrStr)

def getFeatureFactory(featureNode):
    featureType = featureNode.get('type')
    egg, entryPoint = featureType.split(':')
    if egg not in pkg_resources.working_set.by_key:
        ws = easy_install.install([egg], target_dir, newest=True)
        distro = ws.find(pkg_resources.Requirement.parse(eggs))
        pkg_resources.working_set.add(distro)
    else:
        distro = pkg_resources.get_distribution(egg)
    try:
        featureFactory = distro.load_entry_point(
            interfaces.FEATURE_GROUP, entryPoint)
    except ImportError, e:
        raise ValueError("Unable to load feature factory for %s:%s because: %s" % (interfaces.FEATURE_GROUP, entryPoint, e))
    return featureFactory

def getFeaturesDict(node):
    node = getNode(node)
    features = {}
    for featureNode in node.xpath('//feature'):
        featureFactory = getFeatureFactory(featureNode).fromXMLNode(featureNode)
        feature = featureFactory.fromXMLNode(featureNode)
        features[featureNode.get('type')] = feature
    return features

def getFeatures(node):
    return getFeaturesDict(node).values()


def xmlToProject(node):
    node = getNode(node)
    name = node.get("name")
    builder = project.BuildoutProjectBuilder(unicode(name))
    from z3c.feature.core import base
    base.applyFeatures(getFeatures(node), builder)
    builder.add(FeatureDocBuilder(u'ZBOILER.txt'))
    return builder

def extractData(node, iface, convert=True):
    node = getNode(node)
    data = {}
    for fieldName in iface:
        matches = node.xpath('./%s'%fieldName.replace('_','-'))
        match = matches[0] if matches else None
        if match is not None:
            if convert:
                # ignore templated values when converting to actual fields.
                if match.text == "?":
                    continue
                # see if this is a composite element representing a list of
                # items.
                items = match.xpath('./item')
                if items:
                    data[fieldName] = [unicode(item.text) for item in items]
                # otherwise, see if we can parse a value from unicode
                elif hasattr(iface[fieldName], 'fromUnicode'):
                    data[fieldName] = iface[fieldName].fromUnicode(
                        unicode(match.text))
                # oh well, we'll just send back the unicode and hope that's ok
                elif match.text:
                    data[fieldName] = unicode(match.text)
            else:
                data[fieldName] = unicode(match.text)
    return data



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Unit Testing Feature

$Id: unittest.py 98421 2009-03-27 12:42:04Z pcardune $
"""
import os
import zope.interface
from zope.schema.fieldproperty import FieldProperty

from z3c.builder.core import buildout, python
from z3c.builder.core.base import SimpleFileBuilder
from z3c.feature.core import base, interfaces

TESTING_DOCUMENTATION = """
The Automated Testing feature generates boiler plate code for running
unittests, specifically in the form of DocTests.  Here are the various
pieces generated by the Automated Testing Feature:

  1. **new buildout.cfg sections**

       Three new sections are added to the buildout.cfg file. Each
       correspond to executable scripts in the bin/ directory.  There is
       a test runner which you can run with this command::

         $ ./bin/test

       There are many command line options that can be used by the test
       runner.  See them all by running::

         $ ./bin/test --help

       You can also generate coverage reports for the tests with the command::

         $ ./bin/coverage-test

       This runs the same test runner as ``./bin/test`` buts adds on the
       options for generating coverage reports which are always a little
       painful to remember.  By default, this will put all the reports in
       a ``coverage/`` directory. After you have generated the coverage
       reports, you render them into nice color-coded html pages with
       this command::

         $ ./bin/coverage-report

  2. **new setup.py extras_require section**

       In order to support the test runner, a new extras_require section
       called ``test`` is added to the ``setup.py`` file.  This pulls in
       z3c.coverage and zope.testing.

  3. **``tests`` module**

       The test runner will search through your project src for modules
       starting with the name ``test``.  The Automated Testing feature
       adds a sub package named ``tests`` to the project source, and adds
       a python module called ``test_doc.py`` into the tests module.  The
       test_doc module hooks up the test suite that references the doc
       tests.
"""

class TestingFeature(base.BaseFeature):
    zope.interface.implements(interfaces.ITestingFeature)

    iface = interfaces.ITestingFeature

    coverageDirectory = FieldProperty(
        interfaces.ITestingFeature['coverageDirectory'])
    coverageReportDirectory = FieldProperty(
        interfaces.ITestingFeature['coverageReportDirectory'])

    featureTitle = u'Automated Tests'
    featureDocumentation = TESTING_DOCUMENTATION

    def _applyTo(self, project):
        testPartBuilder = buildout.PartBuilder(
            u'test',
            values=[('recipe','zc.recipe.testrunner'),
                    ('eggs',project.projectName+' [test]')])

        project.buildout.add(testPartBuilder)

        coverageTestPart = buildout.PartBuilder(
            u'coverage-test',
            values=[('recipe','zc.recipe.testrunner'),
                    ('eggs',project.projectName+' [test]'),
                    ('defaults',"['--coverage', '%s']" % self.coverageDirectory)])

        project.buildout.add(coverageTestPart)

        coverageReportPart = buildout.PartBuilder(
            u'coverage-report',
            values=[('recipe','zc.recipe.egg'),
                    ('eggs','z3c.coverage'),
                    ('scripts','coverage=coverage-report'),
                    ('arguments',"('coverage', '%s')" % self.coverageReportDirectory)])

        project.buildout.add(coverageReportPart)

        project.setup.addExtrasRequires('test',('zope.testing',
                                                'z3c.coverage'))

        project.package.add(python.PackageBuilder(u'tests'))
        project.package['tests'].add(
            SimpleFileBuilder(u'test_doc.py',
                              template=os.path.join(
                                  os.path.dirname(__file__),
                                  'file-templates',
                                  'test_doc.py')
                              ))

        project.package.add(
            SimpleFileBuilder(u'README.txt',
                              template=os.path.join(
                                  os.path.dirname(__file__),
                                  'file-templates',
                                  'README.txt')
                              ))



##############################################################################
#
# Copyright (c) 2009 Paul Carduner and Stephan Richter.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Setup"""
import os
import xml.sax.saxutils
from setuptools import setup, find_packages

def read(*rnames):
    text = open(os.path.join(os.path.dirname(__file__), *rnames)).read()
    text = unicode(text, 'utf-8').encode('ascii', 'xmlcharrefreplace')
    return xml.sax.saxutils.escape(text)

setup (
    name = 'z3c.feature.core',
    version = '0.1.1',
    author = u"Paul Carduner and Stephan Richter",
    author_email = u"zope-dev@zope.org",
    description = u"Core Features to use with z3c.builder.core",
    long_description=(
        read('README.txt')
        +"\n\n"+
        read('CHANGES.txt')
        ),
    license = "ZPL",
    keywords = u"zope3 project builder feature",
    url = "http://pypi.python.org/pypi/z3c.feature.core",
    packages = find_packages('src'),
    include_package_data = True,
    package_dir = {'':'src'},
    namespace_packages = ['z3c','z3c.feature'],
    extras_require = {
        'test':[
            'zope.testing',
            'z3c.coverage',
            ],
        },
    install_requires = [
        'setuptools',
        'z3c.builder.core',
        ],
    zip_safe = False,
    entry_points = """
    [z3c.boiler.template]
    command-line = z3c.feature.core.template:CommandLineProjectTemplate
    python-package = z3c.feature.core.template:PythonPackageProjectTemplate

    [z3c.feature]
    meta-data = z3c.feature.core.metadata:MetaDataFeature
    comment-header-ZPL = z3c.feature.core.metadata:CommentHeaderZPLFeature
    proprietary-header = z3c.feature.core.metadata:ProprietaryHeaderFeature
    python-interpreter = z3c.feature.core.python:PythonInterpreterFeature
    script = z3c.feature.core.python:ScriptFeature
    unit-testing = z3c.feature.core.unittest:TestingFeature
    documentation = z3c.feature.core.metadata:DocumentationFeature

    [z3c.builderweb]
    meta-data = z3c.feature.core.web.metadata:MetaDataWebFeature
    comment-header-ZPL = z3c.feature.core.web.metadata:CommentHeaderZPLWebFeature
    proprietary-header = z3c.feature.core.web.metadata:ProprietaryHeaderWebFeature
    python-interpreter = z3c.feature.core.web.python:PythonInterpreterWebFeature
    script = z3c.feature.core.web.python:ScriptWebFeature
    unit-testing = z3c.feature.core.web.unittest:TestingWebFeature
    documentation = z3c.feature.core.web.metadata:DocumentationWebFeature
    """,
    )



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test Setup.

$Id: test_doc.py 98421 2009-03-27 12:42:04Z pcardune $
"""
import os
import shutil
import unittest
import tempfile
import logging
import zope.testing.doctest
import zope.component
from zope.configuration import xmlconfig
from z3c.builder.core import testing

def test_suite():
    return unittest.TestSuite((

        zope.testing.doctest.DocFileSuite(
            '../README.txt',
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),

        zope.testing.doctest.DocFileSuite(
            '../example.txt',
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),

        zope.testing.doctest.DocFileSuite(
            '../base.txt',
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),

        zope.testing.doctest.DocFileSuite(
            '../metadata.txt',
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),

        zope.testing.doctest.DocFileSuite(
            '../python.txt',
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),

        zope.testing.doctest.DocFileSuite(
            '../unittest.txt',
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),

        zope.testing.doctest.DocFileSuite(
            '../xml.txt',
            setUp=testing.buildSetUp, tearDown=testing.buildTearDown,
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),

        ))



import zope.schema
from zope.schema.vocabulary import SimpleVocabulary
from zope.schema.vocabulary import SimpleTerm
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile

from z3c.form.form import EditForm
from z3c.form.field import Fields
from z3c.form.browser.textlines import TextLinesFieldWidget

from z3c.feature.core.metadata import MetaDataFeature
from z3c.feature.core.metadata import CommentHeaderZPLFeature
from z3c.feature.core.metadata import ProprietaryHeaderFeature
from z3c.feature.core.metadata import DocumentationFeature
from z3c.feature.core.python import PythonInterpreterFeature
from z3c.feature.core import interfaces

licenseVocabulary = SimpleVocabulary([
    SimpleTerm('Aladdin Free Public License (AFPL)'),
    SimpleTerm('DFSG approved'),
    SimpleTerm('Eiffel Forum License (EFL)'),
    SimpleTerm('Free For Educational Use'),
    SimpleTerm('Free For Home Use'),
    SimpleTerm('Free for non-commercial use'),
    SimpleTerm('Freely Distributable'),
    SimpleTerm('Free To Use But Restricted'),
    SimpleTerm('Freeware'),
    SimpleTerm('Netscape Public License (NPL)'),
    SimpleTerm('Nokia Open Source License (NOKOS)'),
    SimpleTerm('Academic Free License (AFL)'),
    SimpleTerm('Apache Software License'),
    SimpleTerm('Apple Public Source License'),
    SimpleTerm('Artistic License'),
    SimpleTerm('Attribution Assurance License'),
    SimpleTerm('BSD License'),
    SimpleTerm('Common Public License'),
    SimpleTerm('Eiffel Forum License'),
    SimpleTerm('GNU Affero General Public License v3'),
    SimpleTerm('GNU Free Documentation License (FDL)'),
    SimpleTerm('GNU General Public License (GPL)'),
    SimpleTerm('GNU Library or Lesser General Public License (LGPL)'),
    SimpleTerm('IBM Public License'),
    SimpleTerm('Intel Open Source License'),
    SimpleTerm('Jabber Open Source License'),
    SimpleTerm('MIT License'),
    SimpleTerm('MITRE Collaborative Virtual Workspace License (CVW)'),
    SimpleTerm('Motosoto License'),
    SimpleTerm('Mozilla Public License 1.0 (MPL)'),
    SimpleTerm('Mozilla Public License 1.1 (MPL 1.1)'),
    SimpleTerm('Nethack General Public License'),
    SimpleTerm('Nokia Open Source License'),
    SimpleTerm('Open Group Test Suite License'),
    SimpleTerm('Python License (CNRI Python License)'),
    SimpleTerm('Python Software Foundation License'),
    SimpleTerm('Qt Public License (QPL)'),
    SimpleTerm('Ricoh Source Code Public License'),
    SimpleTerm('Sleepycat License'),
    SimpleTerm('Sun Industry Standards Source License (SISSL)'),
    SimpleTerm('Sun Public License'),
    SimpleTerm('University of Illinois/NCSA Open Source License'),
    SimpleTerm('Vovida Software License 1.0'),
    SimpleTerm('W3C License'),
    SimpleTerm('X.Net License'),
    SimpleTerm('zlib/libpng License'),
    SimpleTerm('Zope Public License'),
    SimpleTerm('Other/Proprietary License'),
    SimpleTerm('Public Domain'),
    ])


class MetaDataWebView(EditForm):

    label = u'Project Metadata'
    template = ViewPageTemplateFile('templates/metadata.pt')
    javascript = ViewPageTemplateFile('templates/metadata.js')
    fields = Fields(zope.schema.Choice(
         __name__='commonLicense',
        title=u'License',
        default=u'GNU General Public License (GPL)',
        vocabulary=licenseVocabulary,
        required=False))

    fields += Fields(interfaces.IMetaDataFeature).select(
        'license','version','description','author',
        'author_email','url','classifiers','keywords',
        'namespace_packages','install_requires',
        'commentHeader')

    fields = fields.select(
        'license','commonLicense','version','description','author',
        'author_email','url','classifiers','keywords',
        'namespace_packages','install_requires',
        'commentHeader')

    fields['keywords'].widgetFactory = TextLinesFieldWidget
    fields['namespace_packages'].widgetFactory = TextLinesFieldWidget
    fields['install_requires'].widgetFactory = TextLinesFieldWidget

    def applyChanges(self, data):
        commonLicense = data.pop('commonLicense')
        data['license'] = data.get('license', commonLicense)
        return super(MetaDataWebView, self).applyChanges(data)


class MetaDataWebFeature(object):
    viewFactory = MetaDataWebView
    contentFactory = MetaDataFeature
    title = u"Metadata"
    description = (u"Let's you modify project metadata used by setuptools "
                   u"such as version number, license information, author "
                   u"information, project description, and more.")


class CommentHeaderZPLWebView(EditForm):
    label = u'Comment Header for the Zope Public License'
    fields = Fields(interfaces.ICommentHeaderZPLFeature).select('year')


class CommentHeaderZPLWebFeature(object):
    viewFactory = CommentHeaderZPLWebView
    contentFactory = CommentHeaderZPLFeature
    title = u"ZPL Header"
    description = u"Adds a ZPL Header to all generated files."



class ProprietaryHeaderWebView(EditForm):
    template = ViewPageTemplateFile("templates/proprietary-header.pt")
    fields = Fields(interfaces.IProprietaryHeaderFeature).select(
        'year','company','location')

    def update(self):
        super(ProprietaryHeaderWebView, self).update()
        self.commentHeader = self.context.renderCommentHeader()

class ProprietaryHeaderWebFeature(object):
    viewFactory = ProprietaryHeaderWebView
    contentFactory = ProprietaryHeaderFeature
    title = u"Proprietary Header"
    description = u"Adds a Proprietary Software Header to all generated files."


class DocumentationWebView(EditForm):
    label = u'Documentation'
    fields = Fields(interfaces.IDocumentationFeature)


class DocumentationWebFeature(object):
    viewFactory = DocumentationWebView
    contentFactory = DocumentationFeature
    title = u"Documentation"
    description = (u"Adds a script for generating project documentation from "
                   u"ReSTructured Text files using Sphinx (just like the python "
                   u"documetation).  Great in combination with the "
                   u"Automated Tests Feature")





##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Testing Support

$Id: testing.py 98421 2009-03-27 12:42:04Z pcardune $
"""


class EntryPoint(object):

    def __init__(self, group, name, klass, dist):
        self.group = group
        self.name = name
        self.klass = klass
        self.dist = dist

    def load(self):
        return self.klass

class FeatureDistribution(object):

    project_name = 'z3c.feature.testing'
    key = 'z3c.feature.testing'
    location = 'z3c.feature.testing'

    def __init__(self):
        self.entryPoints = {}

    def addEntryPoint(self, group, name, klass):
        eps = self.entryPoints.setdefault(group, {})
        eps[name] = EntryPoint(group, name, klass, self)

    def get_entry_map(self, group):
        return self.entryPoints[group]

    def insert_on(self, entries, entry):
        entries.append(self.location)

    def activate(self):
        pass
